#!/bin/bash
BRANCH=$1
DEVREPOSITORY=$2
ACCESS_TOKEN=$3
PATCH_PRJECT="Developments"
RODARE_URL="https://rodare.hzdr.de/api/deposit/depositions/" #keep ending slash
UPLOAD_FORMAT=".tgz"
CODEMETA_FILENAME="codemeta.json"
README_FILE="README_RODARE.md"
PYTHON_CMD="python"
DEPOSITION_ID="767" #parent DEPOSITION ID

#Debug via Zenodo Sandbox: https://sandbox.zenodo.org/record/791650
#RODARE_URL="https://sandbox.zenodo.org/api/deposit/depositions/"
#ACCESS_TOKEN="F9PVABEHNqXicRRIJrK5erkwvYQObnKxp1ubsxQaEbV9iV9Wih0fIxvd0Sud"
#DEPOSITION_ID="791668"

#Create a trap to catch errors and get ci job to fail
trap 'exitOnFailure $? $LINENO' ERR

exitOnFailure() {
  echo "Error $1 occurred on $2"
  exit 1
}

#Check that at least codemeta and README file exists
cd ..
if ! [[ -f ${PATCH_PRJECT}/${CODEMETA_FILENAME} && -f ${PATCH_PRJECT}/${README_FILE} ]]; then
    echo "Required ${CODEMETA_FILENAME} or ${README_FILE} doesn't exist."
    exit 1
fi

#Get some hashes to interact with rodare/zenodo api
#Sample response: https://sandbox.zenodo.org/api/deposit/depositions/731451
NEW_DEPOSITION_URL=$(curl -X POST "$RODARE_URL$DEPOSITION_ID/actions/newversion?access_token=$ACCESS_TOKEN" | jq -r '.links.latest_draft')
#Sample response: https://sandbox.zenodo.org/api/files/ecfdb025-74d1-4a9f-ac91-ea64a71c6035
BUCKET_URL=$(curl "$NEW_DEPOSITION_URL?access_token=$ACCESS_TOKEN" | jq -r '.links.bucket')

#Clear all existing files in code publication
FILES=$(curl "$NEW_DEPOSITION_URL?access_token=$ACCESS_TOKEN" | jq -r '.files')
jq -c '.[]' <<<"${FILES}" | while read -r i; do
    _FNAME=$(jq -r .filename <<<"$i")
    curl -X DELETE "${BUCKET_URL}/${_FNAME}?access_token=${ACCESS_TOKEN}"
done

#Create archive for gitlab repository
git clone --depth 1 --branch "${BRANCH}" ${DEVREPOSITORY} "${BRANCH}"
#Remove git and gitlab configuration files
cd "${BRANCH}" && rm -rf .git .gitlab .gitlab-ci.yml && cd ..
tar -zcvf "${BRANCH}${UPLOAD_FORMAT}" "${BRANCH}"
curl --upload-file "${BRANCH}${UPLOAD_FORMAT}" "${BUCKET_URL}/${BRANCH}${UPLOAD_FORMAT}?access_token=${ACCESS_TOKEN}"

#Further steps:
#1. Load codemeta.json file using jq
#2. Iterate over 'targetProduct' to git clone openfoam, third party repos
#   to tar, upload and delete them.
#3. Upload /Developments/README_RODARE.md as README.md
#4. Call python for further metadata processing.
#5. Publish
TRGTPRDCT_DATA=$(jq -r .targetProduct ./${PATCH_PRJECT}/${CODEMETA_FILENAME})
jq -c '.[]' <<<"${TRGTPRDCT_DATA}" | while read -r i; do
    _URL=$(jq -r .downloadUrl <<<"$i")
    _HASH=$(jq -r .softwareVersion <<<"$i")
    _NAME=$(jq -r .name <<<"$i")
    git clone "${_URL}" "__${_NAME}"
    cd "__${_NAME}" && git reset --hard "${_HASH}" && cd ..
    git clone --depth 1 "file://$PWD/__${_NAME}" "${_NAME}"
    tar -zcvf "${_NAME}${UPLOAD_FORMAT}" "${_NAME}"
    curl --upload-file "${_NAME}${UPLOAD_FORMAT}" "${BUCKET_URL}/${_NAME}${UPLOAD_FORMAT}?access_token=${ACCESS_TOKEN}"
    rm -rf "__${_NAME}" "${_NAME}" "${_NAME}${UPLOAD_FORMAT}"
done
curl --upload-file "${PATCH_PRJECT}/${README_FILE}"  "${BUCKET_URL}/README.md?access_token=${ACCESS_TOKEN}"
${PYTHON_CMD} "${PATCH_PRJECT}/.gitlab/bin/rodare_metadata.py" --new_deposition_url "${NEW_DEPOSITION_URL}" --access_token "${ACCESS_TOKEN}" --meta_final "${PATCH_PRJECT}/${CODEMETA_FILENAME}"
curl -i -X POST "${NEW_DEPOSITION_URL}/actions/publish?access_token=${ACCESS_TOKEN}" #publish
