#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 22 11:11:45 2021

@author: ashis
"""

import json
import argparse
import requests

header = {"Content-Type": "application/json"}

def codemeta2zenodo(codemeta_data: dict):
    """

    Parameters
    ----------
    codemeta_data : dict
    Transcodes the codemeta schema to Rodare/Zenodo metadata.
    Returns
    -------
    rodare_metadata : dict
        Metadata object for Rodare/Zenodo.

    """
    c_authors = codemeta_data['author']
    c_contributors = codemeta_data['contributor']
    c_languages = codemeta_data['programmingLanguage']
    c_application_categories = codemeta_data['applicationCategory']
    c_name = codemeta_data['name']
    c_license_id = codemeta_data['license']
    c_description = codemeta_data['description']
    c_keywords = codemeta_data['keywords']
    c_software_version = codemeta_data['softwareVersion']
    c_funder = codemeta_data['funder']
    c_upload_type = codemeta_data['@type']
    c_references = codemeta_data['referencePublication']
    c_lang = 'eng' #or deu

    if 'Software' in c_upload_type:
        c_upload_type = 'software'

    creators = []
    for author in c_authors:
        _temp = {}
        given_name = author['givenName']
        f_name = author['familyName']
        affiliation = author['affiliation']['name']
        _temp['name'] = f_name+', '+given_name
        _temp['affiliation'] = affiliation
        if '@id' in author:
            _temp['orcid'] = author['@id']
        creators.append(_temp)

    contributors = []
    for contributor in c_contributors:
        _temp = {}
        given_name = contributor['givenName']
        f_name = contributor['familyName']
        affiliation = contributor['affiliation']['name']
        _temp['name'] = f_name+', '+given_name
        _temp['affiliation'] = affiliation
        if '@id' in contributor:
            _temp['orcid'] = contributor['@id']
        _temp['type'] = 'Other' #Hard setting every contributor type to 'Other'
        contributors.append(_temp)

    references = []
    for reference in c_references:
        _name = reference['name']
        references.append(_name)

    communities = []
    c_application_categories = c_application_categories.split(',')
    for category in c_application_categories:
        _temp = {}
        _temp['identifier'] = category
        communities.append(_temp)


    rodare_metadata = {
        "metadata": {
            "title": c_name,
            "upload_type": c_upload_type,
            "description": c_description,
            "creators": creators,
            "contributors": contributors,
            "version": c_software_version,
            "keywords": c_keywords + c_languages, \
            #languages appended to keywords
            "references": references,
            "notes": c_funder[0],
            'language': c_lang,
            'license':  c_license_id, #for eg. 'GPL-3.0-only'
            "communities": communities
        }
    }
    return rodare_metadata


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--access_token', required=True)
    parser.add_argument('--new_deposition_url', required=True)
    parser.add_argument('--meta_final', required=True)

    args = parser.parse_args()
    access_token = args.access_token
    deposition_url = args.new_deposition_url
    codemeta_filename = args.meta_final

    with open(codemeta_filename, "r") as read_file:
        codemeta_schema = json.load(read_file)

    rodare_data = codemeta2zenodo(codemeta_schema)
    request = requests.put(deposition_url+'?access_token='+access_token,
                           data=json.dumps(rodare_data), headers=header)

    if request.status_code == 200:
        print('Metadata has been successfully set.')
    else:
        print('An error occurred while setting metadata on', deposition_url)
    print('Status code:', request.status_code)

